﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;

using LumiSoft.Net.RTP;

namespace Rtp_Audio_Demo
{
    /// <summary>
    /// Send PCM audio window.
    /// </summary>
    public class wfrm_SendAudio : Form
    {
        private Label   mt_Codec       = null;
        private TextBox m_pCodec       = null;
        private Label   mt_PacketsSent = null;
        private Label   m_pPacketsSent = null;
        private Label   mt_KBSent      = null;
        private Label   m_pKBSent      = null;

        private wfrm_Main                  m_pMainUI     = null;
        private RTP_Session                m_pSession    = null;
        private string                     m_SendFile    = "";
        private System.Windows.Forms.Timer m_pUITimer    = null;
        private int                        m_PacketsSent = 0;
        private int                        m_BytesSent   = 0;
        private bool                       m_FormClosed  = false;

        /// <summary>
        /// Default constructor.
        /// </summary>
        /// <param name="owner">Main UI.</param>
        /// <param name="session">RTP session.</param>
        /// <param name="sendFile">File which data to send.</param>
        /// <exception cref="ArgumentNullException">Is raised when <b>owner</b>, <b>session</b> or <b>sendFile</b> is null reference.</exception>
        public wfrm_SendAudio(wfrm_Main owner,RTP_Session session,string sendFile)
        {
            if(owner == null){
                throw new ArgumentNullException("owner");
            }
            if(session == null){
                throw new ArgumentNullException("session");
            }
            if(sendFile == null){
                throw new ArgumentNullException("sendFile");
            }

            m_pMainUI  = owner;
            m_pSession = session;
            m_SendFile = sendFile;

            InitUI();

            ThreadPool.QueueUserWorkItem(new WaitCallback(delegate(object state){
                SendAudio();
            }));

            m_pUITimer = new System.Windows.Forms.Timer();
            m_pUITimer.Interval = 500;
            m_pUITimer.Tick += delegate(object s1,EventArgs e1){
                try{
                    m_pCodec.Text       = m_pMainUI.ActiveCodec.Name;
                    m_pPacketsSent.Text = m_PacketsSent.ToString();
                    m_pKBSent.Text      = Convert.ToString(m_BytesSent / 1000);
                }
                catch{
                }
            };
            m_pUITimer.Start();
        }
                
        #region method InitUI

        /// <summary>
        /// Creates and initializes UI.
        /// </summary>
        private void InitUI()
        {
            this.Size = new Size(350,150);
            this.FormBorderStyle = FormBorderStyle.FixedDialog;
            this.MaximizeBox = false;
            this.Text = "Sending audio file: " + Path.GetFileName(m_SendFile);
            this.FormClosing += new FormClosingEventHandler(wfrm_SendAudio_FormClosing);

            mt_Codec = new Label();
            mt_Codec.Location = new Point(0,35);
            mt_Codec.Size = new Size(100,20);
            mt_Codec.Text = "Codec:";
            mt_Codec.TextAlign = ContentAlignment.MiddleRight;

            m_pCodec = new TextBox();
            m_pCodec.Location = new Point(105,35);
            m_pCodec.Size = new Size(100,20);
            m_pCodec.ReadOnly = true;

            mt_PacketsSent = new Label();
            mt_PacketsSent.Location = new Point(0,60);
            mt_PacketsSent.Size = new Size(100,20);
            mt_PacketsSent.Text = "Packets sent:";
            mt_PacketsSent.TextAlign = ContentAlignment.MiddleRight;

            m_pPacketsSent = new Label();
            m_pPacketsSent.Location = new Point(105,60);
            m_pPacketsSent.Size = new Size(100,20);
            m_pPacketsSent.Text = "0";
            m_pPacketsSent.TextAlign = ContentAlignment.MiddleLeft;

            mt_KBSent = new Label();
            mt_KBSent.Location = new Point(0,85);
            mt_KBSent.Size = new Size(100,20);
            mt_KBSent.Text = "KB sent:";
            mt_KBSent.TextAlign = ContentAlignment.MiddleRight;

            m_pKBSent = new Label();
            m_pKBSent.Location = new Point(105,85);
            m_pKBSent.Size = new Size(100,20);
            m_pKBSent.Text = "0";
            m_pKBSent.TextAlign = ContentAlignment.MiddleLeft;

            this.Controls.Add(mt_Codec);
            this.Controls.Add(m_pCodec);
            this.Controls.Add(mt_PacketsSent);
            this.Controls.Add(m_pPacketsSent);
            this.Controls.Add(mt_KBSent);
            this.Controls.Add(m_pKBSent);
        }

        #endregion


        #region Events handling

        #region wfrm_SendAudio_FormClosing

        private void wfrm_SendAudio_FormClosing(object sender,FormClosingEventArgs e)
        {
            if(m_pUITimer != null){
                m_pUITimer.Dispose();
                m_pUITimer = null;
            }

            m_FormClosed = true;
        }

        #endregion

        #endregion


        #region method SendAudio

        /// <summary>
        /// Sends audio to RTP session target(s).
        /// </summary>
        private void SendAudio()
        {
            try{                
                using(FileStream fs = File.OpenRead(m_SendFile)){
                    RTP_SendStream sendStream         = m_pSession.CreateSendStream();
                    int            audioFrameInterval = 20;
                    byte[]         buffer             = new byte[320];
                    int            readedCount        = fs.Read(buffer,0,buffer.Length);
                    int            audioFramesSent    = 0;
                    uint           rtpTimeStamp       = 0;
                    Stopwatch      stopper            = new Stopwatch();
                    stopper.Start();
                    while(!m_FormClosed && readedCount == buffer.Length){
                        if(m_pMainUI.ActiveCodec != null){
                            // We don't have RTP timestamp base or time stamp recycled.
                            if(rtpTimeStamp == 0 || rtpTimeStamp > m_pSession.RtpClock.RtpTimestamp){
                                rtpTimeStamp = m_pSession.RtpClock.RtpTimestamp;
                            }
                            else{
                                rtpTimeStamp += (uint)m_pSession.RtpClock.MillisecondsToRtpTicks(audioFrameInterval);
                            }

                            byte[] encodedData = m_pMainUI.ActiveCodec.Encode(buffer,0,buffer.Length);

                            // Send audio frame.
                            RTP_Packet packet = new RTP_Packet();
                            packet.Timestamp = rtpTimeStamp;
                            packet.Data = encodedData;
                            sendStream.Send(packet);

                            // Read next audio frame.
                            readedCount = fs.Read(buffer,0,buffer.Length);
                            audioFramesSent++;
                            m_BytesSent += encodedData.Length;
                            m_PacketsSent++;
                        }

                        // We are on time sending packet.
                        if(stopper.ElapsedMilliseconds < m_PacketsSent * audioFrameInterval){
                            Thread.Sleep(audioFrameInterval);
                        }
                        // We are missing RTP time for sending packet, get next packet at once.
                        //else{       
                    }
                    sendStream.Close();
                    stopper.Stop();
                }
            }
            catch(Exception x){
                string dummy = x.Message;
            }
        }

        #endregion
    }
}
